@@ -34,7 +34,7 @@ extern "C"
 {
     const char *kss_applicationName = "kanjisaver.kss";
     const char *kss_description = I18N_NOOP( "Kanjisaver" );
-    const char *kss_version = "0.9.9";
+    const char *kss_version = "0.9.10";
 
     kanjisaver *kss_create( WId id )
     {
@@ -102,6 +102,8 @@ void kanjisaverSetup::readSettings()
     DisplayTime = config->readNumEntry("DisplayTime", 10);
 
     switch(ReadingToUse) {
+		case 2: Both->setChecked(true);
+			break;
 		case 1: Kun->setChecked(true);
 			break;
 		case 0: On->setChecked(true);
@@ -109,6 +111,8 @@ void kanjisaverSetup::readSettings()
     };
 
 	switch(TranslationOrReading){
+		case 2: BothTR->setChecked(true);
+			break;
 		case 1: Reading->setChecked(true);
 			break;
 		case 0: Translation->setChecked(true);
@@ -130,9 +134,9 @@ void kanjisaverSetup::slotOkPressed()
 {
     KConfig *config = KGlobal::config();
     config->setGroup( "Settings" );
-    
-    ReadingToUse=(Kun->isChecked()?1:0);
-    TranslationOrReading=(Reading->isChecked()?1:0);
+   
+    if (On->isChecked()) { ReadingToUse=0; } else if (Kun->isChecked()) { ReadingToUse=1; } else { ReadingToUse=2; }
+    if (Translation->isChecked()) { TranslationOrReading=0; } else if (Reading->isChecked()) { TranslationOrReading=1; } else { TranslationOrReading=2; }
     config->writeEntry( "KanaFontFamily", KanaFontSelector->currentFont() );
     config->writeEntry( "KanjiFontFamily", KanjiFontSelector->currentFont() );
     config->writeEntry( "TranslationOrReading", TranslationOrReading);
@@ -366,6 +370,81 @@ void kanjisaver::draw_kanji()
 
 			printf("%s\n", kanjitable[character].pKun);
 	} 
+	else { // Both
+		
+		int text_len = strlen(kanjitable[character].pOn);
+		if (text_len < 20){
+			kanaFont->setPixelSize(myheight/15);
+		}
+		else if (text_len < 28) {
+			kanaFont->setPixelSize(myheight/20);
+		}
+		else if (text_len < 36) {
+			kanaFont->setPixelSize(myheight/26);
+		}
+		else if (text_len < 42) {
+			kanaFont->setPixelSize(myheight/32);
+		}
+		else if (text_len < 49) {
+			kanaFont->setPixelSize(myheight/38);
+		}
+		else if (text_len < 55) {
+			kanaFont->setPixelSize(myheight/43);
+		}
+		else {
+			kanaFont->setPixelSize(myheight/48);
+		}
+		paint.setFont(*kanaFont);
+	
+		paint.drawText( 
+			QRect::QRect(
+					QPoint::QPoint(0 + mywidth/30, myheight/20),
+					QPoint::QPoint(mywidth, myheight/10)
+					),
+			Qt::AlignLeft|Qt::AlignVCenter|Qt::DontClip,
+		   QString::fromUtf8(kanjitable[character].pOn, -1));
+
+		printf("%s\n", kanjitable[character].pOn);
+		
+		// next reading
+
+		text_len = strlen(kanjitable[character].pKun)/3;
+
+		if (text_len < 20){
+			kanaFont->setPixelSize(myheight/15);
+		}
+		else if (text_len < 28) {
+			kanaFont->setPixelSize(myheight/19);
+		}
+		else if (text_len < 37) {
+			kanaFont->setPixelSize(myheight/25);
+		}
+		else if (text_len < 46) {
+			kanaFont->setPixelSize(myheight/30);
+		}
+		else if (text_len < 52) {
+			kanaFont->setPixelSize(myheight/35);
+		}
+		else if (text_len < 57) {
+			kanaFont->setPixelSize(myheight/41);
+		}
+		else {
+			kanaFont->setPixelSize(myheight/47);
+		}
+
+		paint.setFont(*kanaFont);
+
+				paint.drawText( 
+			QRect::QRect(
+					QPoint::QPoint(0 + mywidth/30, myheight/10),
+					QPoint::QPoint(mywidth, myheight/5)
+					),
+			Qt::AlignLeft|Qt::AlignVCenter|Qt::DontClip,
+		   QString::fromUtf8(kanjitable[character].pKun, -1));
+
+			printf("%s\n", kanjitable[character].pKun);
+
+	}
 	
 
 	/// draw the english/romaji
@@ -492,7 +571,279 @@ void kanjisaver::draw_kanji()
 			
 			printf("%s\n", kanjitable[character].pKunRomaji);
 		} 
+		else { // Both
+
+			// font scaling for translation text
+			int text_len = strlen(kanjitable[character].pOnRomaji);
+			if (text_len < 45){
+				romajiFont->setPixelSize(myheight/20);
+			}
+			else if (text_len < 58) {
+				romajiFont->setPixelSize(myheight/24);
+			}
+			else if (text_len < 70) {
+				romajiFont->setPixelSize(myheight/28);
+			}
+			else if (text_len < 78) {
+				romajiFont->setPixelSize(myheight/32);
+			}
+			else if (text_len < 88) {
+				romajiFont->setPixelSize(myheight/35);
+			}
+			else if (text_len < 98) {
+				romajiFont->setPixelSize(myheight/38);
+			}
+			else {
+				romajiFont->setPixelSize(myheight/43);
+			}
+
+			paint.setFont(*romajiFont);
+
+			paint.drawText( 
+					QRect::QRect(
+							QPoint::QPoint(0, 5.5*(myheight/7)),
+							QPoint::QPoint(mywidth - mywidth/30, 6.5*(myheight/7))
+							),
+					Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+				   QString::fromUtf8(kanjitable[character].pOnRomaji, -1));
+		
+			printf("%s\n", kanjitable[character].pOnRomaji);
+
+			// next reading
+
+			text_len = strlen(kanjitable[character].pKunRomaji);
+			if (text_len < 45){
+				romajiFont->setPixelSize(myheight/20);
+			}
+			else if (text_len < 58) {
+				romajiFont->setPixelSize(myheight/24);
+			}
+			else if (text_len < 70) {
+				romajiFont->setPixelSize(myheight/28);
+			}
+			else if (text_len < 78) {
+				romajiFont->setPixelSize(myheight/32);
+			}
+			else if (text_len < 88) {
+				romajiFont->setPixelSize(myheight/35);
+			}
+			else if (text_len < 98) {
+				romajiFont->setPixelSize(myheight/38);
+			}
+			else {
+				romajiFont->setPixelSize(myheight/43);
+			}
+
+			paint.setFont(*romajiFont);
+
+			paint.drawText( 
+					QRect::QRect(
+							QPoint::QPoint(0, 6*(myheight/7)),
+							QPoint::QPoint(mywidth - mywidth/30, myheight)
+							),
+					Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+				   QString::fromUtf8(kanjitable[character].pKunRomaji, -1));
+			
+			printf("%s\n", kanjitable[character].pKunRomaji);
+
+		}
 	}
+	else { // Both Translation and Reading
+	// FIXME: this really proves the need for functions :- /
+	
+		int text_len = strlen(kanjitable[character].pEnglish);
+		if (text_len < 45){
+			romajiFont->setPixelSize(myheight/20);
+		}
+		else if (text_len < 58) {
+			romajiFont->setPixelSize(myheight/24);
+		}
+		else if (text_len < 70) {
+			romajiFont->setPixelSize(myheight/28);
+		}
+		else if (text_len < 78) {
+			romajiFont->setPixelSize(myheight/32);
+		}
+		else if (text_len < 88) {
+			romajiFont->setPixelSize(myheight/35);
+		}
+		else if (text_len < 98) {
+			romajiFont->setPixelSize(myheight/38);
+		}
+		else {
+			romajiFont->setPixelSize(myheight/43);
+		}
+		
+		paint.setFont(*romajiFont);
+
+			paint.drawText( 
+				QRect::QRect(
+					QPoint::QPoint(0, 6*(myheight/7)),
+					QPoint::QPoint(mywidth - mywidth/30, myheight)
+					),
+					Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+		   QString::fromUtf8(kanjitable[character].pEnglish, -1));
+			
+
+			printf("%s\n", kanjitable[character].pEnglish);
+
+		if (ReadingToUse == 0) { // On
+
+		// font scaling for translation text
+		int text_len = strlen(kanjitable[character].pOnRomaji);
+		if (text_len < 45){
+			romajiFont->setPixelSize(myheight/20);
+		}
+		else if (text_len < 58) {
+			romajiFont->setPixelSize(myheight/24);
+		}
+		else if (text_len < 70) {
+			romajiFont->setPixelSize(myheight/28);
+		}
+		else if (text_len < 78) {
+			romajiFont->setPixelSize(myheight/32);
+		}
+		else if (text_len < 88) {
+			romajiFont->setPixelSize(myheight/35);
+		}
+		else if (text_len < 98) {
+			romajiFont->setPixelSize(myheight/38);
+		}
+		else {
+			romajiFont->setPixelSize(myheight/43);
+		}
+
+		paint.setFont(*romajiFont);
+
+		paint.drawText( 
+				QRect::QRect(
+						QPoint::QPoint(0, 5.5*(myheight/7)),
+						QPoint::QPoint(mywidth - mywidth/30, 6.5*(myheight/7))
+						),
+				Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+			   QString::fromUtf8(kanjitable[character].pOnRomaji, -1));
+	
+		printf("%s\n", kanjitable[character].pOnRomaji);
+	
+	}
+	else if (ReadingToUse == 1) { // Kun
+
+		// font scaling for translation text
+		int text_len = strlen(kanjitable[character].pKunRomaji);
+		if (text_len < 45){
+			romajiFont->setPixelSize(myheight/20);
+		}
+		else if (text_len < 58) {
+			romajiFont->setPixelSize(myheight/24);
+		}
+		else if (text_len < 70) {
+			romajiFont->setPixelSize(myheight/28);
+		}
+		else if (text_len < 78) {
+			romajiFont->setPixelSize(myheight/32);
+		}
+		else if (text_len < 88) {
+			romajiFont->setPixelSize(myheight/35);
+		}
+		else if (text_len < 98) {
+			romajiFont->setPixelSize(myheight/38);
+		}
+		else {
+			romajiFont->setPixelSize(myheight/43);
+		}
+
+		paint.setFont(*romajiFont);
+
+		paint.drawText( 
+				QRect::QRect(
+						QPoint::QPoint(0, 5.5*(myheight/7)),
+						QPoint::QPoint(mywidth - mywidth/30, 6.5*(myheight/7))
+						),
+				Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+			   QString::fromUtf8(kanjitable[character].pKunRomaji, -1));
+		
+		printf("%s\n", kanjitable[character].pKunRomaji);
+	} 
+	else { // Both
+
+		// font scaling for translation text
+		int text_len = strlen(kanjitable[character].pOnRomaji);
+		if (text_len < 45){
+			romajiFont->setPixelSize(myheight/20);
+		}
+		else if (text_len < 58) {
+			romajiFont->setPixelSize(myheight/24);
+		}
+		else if (text_len < 70) {
+			romajiFont->setPixelSize(myheight/28);
+		}
+		else if (text_len < 78) {
+			romajiFont->setPixelSize(myheight/32);
+		}
+		else if (text_len < 88) {
+			romajiFont->setPixelSize(myheight/35);
+		}
+		else if (text_len < 98) {
+			romajiFont->setPixelSize(myheight/38);
+		}
+		else {
+			romajiFont->setPixelSize(myheight/43);
+		}
+
+		paint.setFont(*romajiFont);
+
+		paint.drawText( 
+				QRect::QRect(
+						QPoint::QPoint(0, 5*(myheight/7)),
+						QPoint::QPoint(mywidth - mywidth/30, 6*(myheight/7))
+						),
+				Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+			   QString::fromUtf8(kanjitable[character].pOnRomaji, -1));
+	
+		printf("%s\n", kanjitable[character].pOnRomaji);
+
+		// next reading
+
+		text_len = strlen(kanjitable[character].pKunRomaji);
+		if (text_len < 45){
+			romajiFont->setPixelSize(myheight/20);
+		}
+		else if (text_len < 58) {
+			romajiFont->setPixelSize(myheight/24);
+		}
+		else if (text_len < 70) {
+			romajiFont->setPixelSize(myheight/28);
+		}
+		else if (text_len < 78) {
+			romajiFont->setPixelSize(myheight/32);
+		}
+		else if (text_len < 88) {
+			romajiFont->setPixelSize(myheight/35);
+		}
+		else if (text_len < 98) {
+			romajiFont->setPixelSize(myheight/38);
+		}
+		else {
+			romajiFont->setPixelSize(myheight/43);
+		}
+
+		paint.setFont(*romajiFont);
+
+		paint.drawText( 
+				QRect::QRect(
+						QPoint::QPoint(0, 5.5*(myheight/7)),
+						QPoint::QPoint(mywidth - mywidth/30, 6.5*(myheight/7))
+						),
+				Qt::AlignRight|Qt::AlignVCenter|Qt::DontClip,
+			   QString::fromUtf8(kanjitable[character].pKunRomaji, -1));
+		
+		printf("%s\n", kanjitable[character].pKunRomaji);
+
+	}
+		
+			
+	}
+
 
 	printf("----------------------\n");
 	timer->changeInterval(DisplayTime*1000);
